@@ -19,6 +19,7 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 import numpy
 
 from pyrr import vector
+from pyrr.utils import all_parameters_as_numpy_arrays, parameters_as_numpy_arrays
 
 
 class index:
@@ -29,37 +30,44 @@ class index:
     direction = 1
 
 
-def create_identity():
+def create_identity(dtype=None):
     return numpy.array(
         [
             [ 0.0, 0.0, 0.0 ],
             [ 0.0, 0.0,-1.0 ]
-            ]
-        )
+        ],
+        dtype=dtype
+    )
 
-def create_ray( start, direction ):
+@parameters_as_numpy_arrays('start', 'direction')
+def create_ray(start, direction, dtype=None):
+    dtype = dtype or start.dtype
     return numpy.array(
         [
             start,
-            vector.normalise( direction )
-            ]
-        )
+            vector.normalise(direction)
+        ],
+        dtype=dtype
+    )
 
-def create_from_line( line ):
+@parameters_as_numpy_arrays('line')
+def create_from_line(line, dtype=None):
     """
     Converts a line or line segment to a ray.
     """
+    dtype = dtype or line.dtype
     # direction = vend - vstart
     return numpy.array(
         [
-            line[ 0 ],
-            vector.normalise( line[ 1 ] - line[ 0 ] )
-            ]
-        )
+            line[0],
+            vector.normalise(line[1] - line[0])
+        ],
+        dtype=dtype
+    )
 
-def origin( ray ):
-    return ray[ 0 ]
+def origin(ray):
+    return ray[0].copy()
 
-def direction( ray ):
-    return ray[ 1 ]
+def direction(ray):
+    return ray[1].copy()
 
