@@ -2,25 +2,28 @@
 """Provides functions for creating and manipulating 4D vectors.
 """
 import numpy
+from pyrr.utils import parameters_as_numpy_arrays
 
 
-def create_identity():
-    return numpy.array( [ 0.0, 0.0, 0.0, 1.0 ] )
+def create_identity(dtype=None):
+    return numpy.array([0.0, 0.0, 0.0, 1.0], dtype=dtype)
 
-def create_unit_length_x():
-    return numpy.array( [ 1.0, 0.0, 0.0, 1.0 ] )
+def create_unit_length_x(dtype=None):
+    return numpy.array([1.0, 0.0, 0.0, 1.0], dtype=dtype)
 
-def create_unit_length_y():
-    return numpy.array( [ 0.0, 1.0, 0.0, 1.0 ] )
+def create_unit_length_y(dtype=None):
+    return numpy.array([0.0, 1.0, 0.0, 1.0], dtype=dtype)
 
-def create_unit_length_z():
-    return numpy.array( [ 0.0, 0.0, 1.0, 1.0 ] )
+def create_unit_length_z(dtype=None):
+    return numpy.array([0.0, 0.0, 1.0, 1.0], dtype=dtype)
 
-def create_from_vector3( vector ):
-    return numpy.array( [ vec[0], vec[1], vec[2], 1.0 ] )
+@parameters_as_numpy_arrays('vector')
+def create_from_vector3(vector, dtype=None):
+    dtype = dtype or vector.dtype
+    return numpy.array([vec[0], vec[1], vec[2], 1.0], dtype=dtype)
 
-def create_from_matrix44_translation( mat ):
-    return mat[ 3, 0:4 ].copy()
+def create_from_matrix44_translation(mat):
+    return mat[3, 0:4].copy()
 
 
 class index:
